/*********************** SCRCLEAR.C ********************** 
* Determines presence of EGA/VGA and the correct number of
* columns and rows. Using these data, it clears the screen
* Usage: rowcol
* (c) 1988 Nigel Cort / C Gazette. Use freely but acknow-
* ledge source. Small case. Version 2.2
*********************************************************/

/* Verified for Datalight, Ecosoft, MSC, Turbo C, Watcom */

#include <stdio.h>
#include <dos.h>

#define DISPLAY  1      /* Demo: to show results         */

                        /* --- Display Types --- */
#define VGA     2
#define EGA     1
#define NEITHER 0           /* Not VGA or EGA */

#define ALPHA 0x07

unsigned int rows, cols, mode, page;

union REGS inregs, outregs;

int  ega_or_vga(void);
void get_cols_mode_page(void),
     get_rows(void);

void get_cols_mode_page (void)  /* Uses service 0Fh of video interrupt */
{                               /* to extract the named data items     */
    inregs.h.ah = 0x0F;
    int86 (0x10, &inregs, &outregs);
    cols = outregs.h.ah;
    mode = outregs.h.al;
    page = outregs.h.bh;

#ifdef DISPLAY
    printf ("\n\n\n\n\Columns = %u, mode = %u, page = %u ",
             cols, mode, page);
#endif
}

void get_rows (void)            /* Extract row count, consult text on this */
{
    if (ega_or_vga())           /* If an EGA or VGA is in use,  */
    {                           /* you should use the character */
        inregs.h.ah = 0x11;     /* generator BIOS call to get   */
        inregs.h.al = 0x30;     /* the number of rows.          */
        int86 (0x10, &inregs, &outregs);
        rows = outregs.h.dl + 1;
    }
    else
        rows = 25;              /* Mono, CGA are coerced to 25  */

#ifdef DISPLAY
    printf ("Rows = %u\n", rows);
    getchar();
#endif
}

int ega_or_vga()    /* Returns non-zero if EGA or VGA is installed */
{
    inregs.h.ah = 0x1A;     /* service to get VGA monitor info    */
    inregs.h.al = 0x00;
    int86 (0x10, &inregs, &outregs);

    if (outregs.x.bx != 0)  /* BL = 7 for mono VGA, 8 = color VGA */
    {                       /* BX = 0 this VGA call not supported */
#if DISPLAY
        puts ("VGA Found");
#endif
        return (VGA);
    }

    inregs.h.ah = 0x12;     /* service to get EGA monitor info    */
    inregs.h.bl = 0x10;
    int86 (0x10, &inregs, &outregs);

    if (outregs.h.bl != 0x10)   /* if EGA, returns 0-3 depending  */
    {                           /* on memory size, else it's 0x10 */
#if DISPLAY
        puts ("EGA found");
#endif
        return (EGA);
    }

#if DISPLAY
    puts ("NEITHER Found");
#endif
    return (NEITHER);       /* No EGA or VGA if we got this far   */
}

main()      /* Uses the extracted information to clear the screen */
{

    get_cols_mode_page();
    get_rows();

    /* now clear the screen using service 6 of int 10h
       which scrolls up for set number of rows & columns */

    inregs.h.dl = --cols;
    inregs.h.dh = --rows;
    inregs.h.cl = 0;
    inregs.h.ch = 0;
    inregs.h.bh = (mode <= ALPHA ? ALPHA : 0); /* attributes */
    inregs.h.al = 0;   /* number of lines to scroll, 0 = whole screen */
    inregs.h.ah = 6;   /* the upward scrolling service */
    int86 (0x10, &inregs, &outregs);

    /* reset the cursor to 0, 0 */

    inregs.h.dh = 0;    /* row 0 */
    inregs.h.dl = 0;    /* col 0 */
    inregs.h.bh = page; /* page  */
    inregs.h.ah = 2;    /* DOS service to place the cursor */
    int86 (0x10, &inregs, &outregs);

    return (0);
}

