/******************** SOUNDEX.C ***********************
*   Will generate the soundex code for a name or word.
*   by C Gazette Technical Staff (c) Copyright 1988
*   Use freely but acknowledge source.
******************************************************/

/* Verified for: Datalite, DeSmet, Turbo, MetaWare, MSC, Watcom */

#include <ctype.h>
#include <string.h>

#define DRIVER 1        /*  Toggle for main() to demo usage */

int soundex (char *str, char *soundex_out)
{

#define SOUNDEX_ERR 0
#define SOUNDEX_OK  1

    static const char table [27] =

      /* ABCDEFGHIJKLMNOPQRSTUVWXYZ */
        "01230120022455012623010202";

    int count = 0;                  /* Valid code ltrs */
    char this_char, last_char;      /* Current & previous */
    char code [5];

    strcpy (code, "    ");          /* Initialize string */

    this_char = toupper(*str);      /* The first character is preserved */
    if (isalpha(this_char))         /* albeit as upper-case             */
    {
        code[0] = this_char;
        count = 1;
        str++;
    }
    else
        return (SOUNDEX_ERR);

    last_char = ' ';               /* Last char empty at this point   */

    while (count < 4 && isalpha(*str))
    {
        char c;

        this_char = toupper (*str);
        c = table[this_char - 'A'];

        if (c != last_char && c != '0')    /* Remove repeated sequences */
        {                                  /* and 0's from output       */
            code[count++] = c;             /* Then insert it into string*/
            last_char = c;
        }
        str += 1;
    }

    if (*str != '\0' && !(isalpha(*str)))           /* If it's not EOS   */
        return (SOUNDEX_ERR);                       /* nor a letter,     */
    else                                            /* it's an error.    */
    {
        strcpy (soundex_out, code);
        return (SOUNDEX_OK);
    }
}

#ifdef DRIVER

#include <stdio.h>    /* Driver to demo soundex results */

main ()
{
     char name [80], code[5];

     puts ("Enter a name or word for which a Soundex is needed: ");
     gets (name);

     if (soundex (name, code) == 0)
        printf ("%s is not a valid name or word\n", name);
     else
        printf ("Soundex for %s is %s\n", name, code);

     return (1);
}
#endif 
